"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bridge_serializer_1 = require("./bridge-serializer");
const DEFAULT_BRIDGE_TRANSFER_OPTIONS = {
    release: true,
    transfer: false
};
class ActiveBridge {
    constructor(dispatch, ivm) {
        this.dispatch = dispatch;
        this.ivm = ivm;
    }
    externalCopy(arg) {
        return new this.ivm.ExternalCopy(arg);
    }
    call(fn, ...args) {
        return this.apply(this.dispatch, fn, args);
    }
    callIgnored(fn, ...args) {
        return this.applyIgnored(this.dispatch, fn, args);
    }
    callSync(fn, ...args) {
        return this.applySync(this.dispatch, fn, args);
    }
    apply(receiver, fn, args) {
        return receiver.apply(undefined, [fn, ...args.map(a => this.wrap(a))]);
    }
    applySync(receiver, fn, args) {
        return receiver.applySync(undefined, [fn, ...args.map(a => this.wrap(a))]);
    }
    applyIgnored(receiver, ...args) {
        return receiver.applyIgnored(undefined, args.map(a => this.wrap(a)));
    }
    wrap(arg, opts = {}) {
        if (!arg) {
            return arg;
        }
        if (arg instanceof Error) {
            const transformedErrorObj = {
                name: arg.name || '',
                message: arg.message || '',
                stack: arg.stack || ''
            };
            return this.wrapValue(transformedErrorObj, { release: !!opts.release, transfer: false });
        }
        if (arg instanceof bridge_serializer_1.BridgeSerializer) {
            arg = arg.serialize((arg) => this.externalCopy(arg), (fn, opts) => this.wrapFunction(fn, opts));
        }
        let ctor;
        try {
            ctor = arg.constructor;
        }
        catch (e) {
            ctor = undefined;
        }
        switch (ctor) {
            case this.ivm.ExternalCopy:
            case this.ivm.Reference:
            case String:
            case Number:
            case Boolean:
                return arg;
            case ArrayBuffer:
            case Uint8Array:
            case Int8Array:
            case Uint8ClampedArray:
            case Int16Array:
            case Uint16Array:
            case Int32Array:
            case Uint32Array:
            case Float32Array:
            case Float64Array:
                return this.wrapValue(arg, opts);
            case DataView:
            case Map:
            case Set:
            case RegExp:
            case Date:
            case Object:
                return this.wrapValue(arg, { release: !!opts.release, transfer: false });
            case Array:
                return this.wrapArray(arg, { release: !!opts.release, transfer: false });
            case Function:
                return this.wrapFunction(arg);
            default:
                throw new Error(`Can't wrap a non-transferable value (constructor: '${(ctor && ctor.name) || "unknown"}')`);
        }
    }
    wrapFunction(fn, options = { release: true }) {
        const opts = Object.assign({}, DEFAULT_BRIDGE_TRANSFER_OPTIONS, options || {});
        if (!opts.release) {
            return new this.ivm.Reference(fn);
        }
        const cb = new this.ivm.Reference(function bridgeAutoReleaseFn(...args) {
            try {
                cb.release();
            }
            catch (err) {
            }
            finally {
                fn(...args);
            }
        });
        return cb;
    }
    wrapValue(value, options = { release: true }) {
        const opts = Object.assign({}, DEFAULT_BRIDGE_TRANSFER_OPTIONS, options || {});
        if (!!opts.transfer) {
            return new this.ivm.ExternalCopy(value, { transferOut: true }).copyInto({
                release: !!opts.release,
                transferIn: true
            });
        }
        return new this.ivm.ExternalCopy(value).copyInto({ release: !!opts.release });
    }
    wrapArray(value, options = { release: true }) {
        return this.wrapValue(value.map(v => this.wrap(v)), options);
    }
}
exports.ActiveBridge = ActiveBridge;
