"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const abstract_definition_1 = require("./abstract-definition");
const util_1 = require("../../util");
class ApiDefinition extends abstract_definition_1.AbstractDefinition {
    constructor() {
        super(...arguments);
        this.requestJira = (url, init) => {
            return util_1.apiFetch(this.bridge, 'api.requestJira', url, init);
        };
        this.requestConfluence = (url, init) => {
            return util_1.apiFetch(this.bridge, 'api.requestConfluence', url, init);
        };
        this.fetch = (url, init) => {
            return util_1.apiFetch(this.bridge, 'api.fetch', url, init);
        };
        this.asUser = () => {
            return {
                requestJira: (url, init) => util_1.apiFetch(this.bridge, 'api.asUser.requestJira', url, init),
                requestConfluence: (url, init) => util_1.apiFetch(this.bridge, 'api.asUser.requestConfluence', url, init)
            };
        };
        this.asApp = () => {
            return {
                requestJira: (url, init) => util_1.apiFetch(this.bridge, 'api.asApp.requestJira', url, init),
                requestConfluence: (url, init) => util_1.apiFetch(this.bridge, 'api.asApp.requestConfluence', url, init)
            };
        };
        this.isUnexpectedLookupError = (versionResponse) => {
            const expectedError = 404;
            return !versionResponse.ok && versionResponse.status !== expectedError;
        };
        this.isUnexpectedGetError = (versionResponse) => {
            const expectedErrors = [400, 401, 403, 404];
            return !versionResponse.ok && !expectedErrors.includes(versionResponse.status);
        };
        this.getUpdatedVersion = async (versionResponse) => {
            if (!versionResponse.ok) {
                return 1;
            }
            else {
                const data = await versionResponse.json();
                return data.version.number + 1;
            }
        };
        this.buildSetRequestBody = (value, updatedVersion) => {
            return {
                value: value,
                version: { number: updatedVersion }
            };
        };
        this.buildSetRequestOptions = (requestBody, usePut = true) => {
            const method = usePut ? 'PUT' : 'POST';
            return {
                method: method,
                'content-type': 'application/json',
                body: JSON.stringify(requestBody)
            };
        };
        this.checkUserInput = (context, property) => {
            const regex = /^[A-Za-z0-9_\-]+$/;
            if (!regex.test(context)) {
                throw new Error(`Invalid context key`);
            }
            if (!regex.test(property)) {
                throw new Error(`Invalid property key`);
            }
        };
        this.storeMethods = (prefix, context, product, method) => {
            console.warn('\x1b[33m%s\x1b[0m', `[store.${method}] Your current version of @forge/api is deprecated. Run 'npm i @forge/api@latest' to prevent future breaking changes. https://developer.atlassian.com/platform/forge/changelog`);
            const queryProduct = (key, options = {}) => {
                this.checkUserInput(context, key);
                switch (product) {
                    case util_1.Product.jira:
                        return this.asApp().requestJira(`${prefix}${context}/properties/${key}`, options);
                    case util_1.Product.confluence:
                        return this.asApp().requestConfluence(`${prefix}${context}/property/${key}`, options);
                }
            };
            const getHandler = async (key) => {
                const response = await queryProduct(key);
                if (this.isUnexpectedGetError(response)) {
                    util_1.handleAPIErrors(response.status, 'GET');
                }
                else if (!response.ok) {
                    return undefined;
                }
                const { value } = await response.json();
                return value;
            };
            const setHandlerJira = async (key, value) => {
                const requestOptions = this.buildSetRequestOptions(value);
                const response = await queryProduct(key, requestOptions);
                if (!response.ok) {
                    util_1.handleAPIErrors(response.status, 'PUT');
                }
            };
            const setHandlerConfluence = async (key, value) => {
                const versionResponse = await queryProduct(key);
                if (this.isUnexpectedLookupError(versionResponse)) {
                    util_1.handleAPIErrors(versionResponse.status, 'GET');
                }
                const updatedVersion = await this.getUpdatedVersion(versionResponse);
                const requestBody = this.buildSetRequestBody(value, updatedVersion);
                const requestOptions = this.buildSetRequestOptions(requestBody, versionResponse.ok);
                const response = await queryProduct(key, requestOptions);
                if (!response.ok) {
                    util_1.handleAPIErrors(response.status, 'PUT');
                }
            };
            const deleteHandler = async (key) => {
                const response = await queryProduct(key, { method: 'delete' });
                if (!response.ok) {
                    util_1.handleAPIErrors(response.status, 'DEL');
                }
            };
            const storeMethods = {
                get: getHandler,
                delete: deleteHandler
            };
            switch (product) {
                case util_1.Product.jira:
                    return { ...storeMethods, set: setHandlerJira };
                case util_1.Product.confluence:
                    return { ...storeMethods, set: setHandlerConfluence };
            }
        };
        this.getAuthTokenReferences = (serviceKey) => {
            return this.bridge.callSync('api.getAuthTokenReferences', serviceKey);
        };
    }
    definePreBootstrap(global) {
        global.api = {
            asUser: this.asUser,
            asApp: this.asApp,
            fetch: this.fetch,
            requestJira: this.requestJira,
            requestConfluence: this.requestConfluence,
            store: {
                onJiraIssue: (context) => {
                    const JIRA_ISSUE_PREFIX = `/rest/api/3/issue/`;
                    return this.storeMethods(JIRA_ISSUE_PREFIX, context, util_1.Product.jira, 'onJiraIssue');
                },
                onJiraProject: (context) => {
                    const JIRA_ISSUE_PREFIX = `/rest/api/3/project/`;
                    return this.storeMethods(JIRA_ISSUE_PREFIX, context, util_1.Product.jira, 'onJiraProject');
                },
                onConfluencePage: (context) => {
                    const CONFLUENCE_PAGE_PREFIX = `/wiki/rest/api/content/`;
                    return this.storeMethods(CONFLUENCE_PAGE_PREFIX, context, util_1.Product.confluence, 'onConfluencePage');
                },
                onConfluenceSpace: (context) => {
                    const CONFLUENCE_SPACE_PREFIX = `/wiki/rest/api/space/`;
                    return this.storeMethods(CONFLUENCE_SPACE_PREFIX, context, util_1.Product.confluence, 'onConfluenceSpace');
                }
            }
        };
    }
    definePostBootstrap(global, options) {
        if (options && options.FFisMultiAuthEnabled) {
            global.api.getAuthTokenReferences = this.getAuthTokenReferences;
        }
    }
}
exports.ApiDefinition = ApiDefinition;
